# Python 3
# Author: Hans Durchholz

from os import getcwd
from socket import socket, AF_INET, SOCK_STREAM
import sys

ERROR_ARGS = "\nPlease try running the file again with proper arguments."


if __name__ == "__main__":
    try:
        if len(sys.argv) == 3:
            server_host = sys.argv[1]
            server_port = int(sys.argv[2])
        elif len(sys.argv) == 1:
            server_host = input("Host Address: ")
            server_port = int(input("Port: "))
        else:
            print("\tExpected CLI arguments: 2\n\tGiven: {}\n".
                  format(len(sys.argv)))
            sys.exit(ERROR_ARGS)
    except ValueError as e:
        print("Please input a valid port. You inputted: {}".
              format(str(e)[str(e).find("'"):]))
        sys.exit(ERROR_ARGS)

    # Create a TCP server socket
    server_socket = socket(AF_INET, SOCK_STREAM)
    server_socket.bind((server_host, server_port))
    server_socket.listen(1)
    print("Listening on port {}...".format(server_port))

    while True:
        print("The server is ready to receive")

        connection_socket, address = server_socket.accept()

        try:
            message = connection_socket.recv(1024).decode()

            if len(message.split()) <= 1:   # ignore empty response
                continue

            print("RESPONSE:\n{}".format(message))

            filename = message.split()[1]

            if filename[1:] != "favicon.ico":
                with open(filename[1:]) as f:
                    output_data = f.read()
            else:
                with open(filename[1:], 'rb') as f:
                    output_data = f.read()
                    output_data = output_data.decode('iso-8859-1')
            # Send one HTTP header line into socket
            connection_socket.send("HTTP/1.1 200 OK\r\n\r\n".encode())

            # Send the content of the requested file to the client
            for i in range(len(output_data)):
                if filename[1:] != "favicon.ico":
                    connection_socket.send(output_data[i].encode())
                else:
                    connection_socket.send(output_data[i].encode('iso-8859-1'))

            connection_socket.send("\r\n".encode())

            # Close the client connection socket
            connection_socket.close()
        except IOError as e:
            # Send response message for file not found
            connection_socket.send("404 Not Found - {} in \'{}\'".format(
                str(e)[str(e).find(']') + 2:], getcwd()).encode())

            # Close the client connection socket
            connection_socket.close()

    server_socket.close()
    sys.exit()
